@@ -86,14 +86,13 @@
 
 void *zmalloc(int64 cc, ch *tag);                                                //  malloc() wrapper
 void zfree(void *pp);                                                            //  free() wrapper
-void zmalloc_report(void *zd);                                                   //  memory usage by tag > popup report
-void zmalloc_growth(void *zd);                                                   //  memory growth by tag > popup report
 int  zmalloc_test(int64 cc);                                                     //  test if cc free memory available
 double realmemory();                                                             //  get available real memory in MB units
 double availmemory();                                                            //  get available memory (+swap) in MB units
 ch * zstrdup(ch *zstring, ch *tag, int addcc = 0);                               //  strdup() wrapper with added space
-int  zstrcopy(ch *&zstring, ch *string, ch *tag, int addcc = 0);                 //  replace zstring with string + added cc
-void Plog(int lev, ch *format, ...);                                             //  add message to stdout log file, fflush()
+int  zstrcopy(ch *&zstring, ch *string, ch *tag, int addcc = 0);                 //  replace string with new string + added cc
+void zmalloc_report(GtkWidget *parent);                                          //  memory usage by tag > popup report
+void zmalloc_growth(GtkWidget *parent);                                          //  memory growth by tag > popup report
 void xmessage(ch *message);                                                      //  popup message not needing GTK
 void zexit(int popup, ch *message, ...);                                         //  exit a process and kill child processes
 void zbacktrace();                                                               //  produce a backtrace to stdout
@@ -101,20 +100,17 @@ void zappcrash(ch *format, ...);
 void catch_signals();                                                            //  catch signals and do backtrace dump
 void trace(ch *file, ch *func, int line, void *addr);                            //  implements TRACE macro
 void tracedump();                                                                //  dump program trace data
-double get_seconds();                                                            //  real seconds since 2000.01.01
+ch * combine_argvs(int argc, ch *argv[], int Nth);                               //  combine argv[ii] elements Nth to last
+double get_seconds(int init = 0);                                                //  get elapsed seconds since last call
+void print_seconds(ch *label);                                                   //  print seconds since last call
 double get_seconds(timespec *ts);                                                //  real seconds from timespec struct
-double get_modtime(char *file);                                                  //  file mod time in real seconds
-void logtime_init(ch *text);                                                     //  set initial time
-void logtime(ch *text);                                                          //  log elapsed time since prior call
+int zstat(ch *file, uint &mtime, uint &size);                                    //  get file mod time (secs-1970) and size
 void start_timer(double &time0);                                                 //  start a timer
 double get_timer(double &time0);                                                 //  get elapsed time in seconds
-void start_CPUtimer(double &time0);                                              //  start a process CPU timer
-double get_CPUtimer(double &time0);                                              //  get elapsed CPU time, seconds
 double CPUtime();                                                                //  get elapsed process CPU time
 int  memused();                                                                  //  get process real memory, MB
-int  proc_resources(double &cpu, double &bio);                                   //  get process CPU and block IO counts
 void compact_time(time_t DT, ch *compactDT);                                     //  time_t DT to yyyymmddhhmmss
-void pretty_datetime(time_t DT, ch *prettyDT);                                   //  time_t DT to yyyy-mm-dd hh:mm:ss
+void pretty_datetime(time_t DT, ch *prettyDT);                                   //  time_t DT to yyyy:mm:dd hh:mm:ss
 void secs_datetime(double secs, int datetime[6]);                                //  seconds to year/mon/day/hour/min/secs
 void datetime_secs(int datetime[6], double *secs);                               //  year/mon/day/hour/min/secs to seconds
 int  parseprocfile(ch *pfile, ch *pname, double *value, ...);                    //  get data from /proc file
@@ -136,17 +132,16 @@ pthread_t start_Jthread(void * threadfunc(void *), void * arg);
 int  wait_Jthread(pthread_t tid);                                                //  wait for completion (join thread)
 void synch_threads(int NT = 0);                                                  //  synchronize NT threads
 int  main_thread();                                                              //  return 1 if main() thread, else 0
-void set_cpu_affinity(int cpu);                                                  //  set cpu affinity for calling thread
 int  zshell(ch *options, ch *command, ...);                                      //  do shell command and get status
-ch * command_output(int &contx, ch *command, ...);                               //  get shell command output
-int  command_status(int contx);                                                  //  get exit status of command
-int  signalProc(ch * pname, ch * signal);                                        //  pause/resume/kill subprocess
-ch * fgets_trim(ch * buff, int maxcc, FILE *, int bf = 0);                       //  fgets + trim trailing \n \r (blanks)
+int  kill_procname(ch *wildname);                                                //  kill processes matching wilcard name
+int  signalProc(ch *pname, ch *signal);                                          //  send signal to process by process name
+ch * fgets_trim(ch * buff, int bcc, FILE *, int bf = 0);                         //  fgets() + trim trailing \n \r (blanks)
+ch * fgets_pend(ch * buff, int bcc, FILE *);                                     //  fgets() non-blocking wrapper
 int  samefolder(ch *file1, ch *file2);                                           //  returns 1 if files in same folder
-int  parsefile(ch *path, ch **dir, ch **file, ch **ext);                         //  parse a filespec
+int  parsefile(ch *path, ch *&folder, ch *&file, ch *&ext);                      //  parse components of a file name
+void parsefile_free(ch *&folder, ch *&file, ch *&ext);                           //  zfree file name components
 int  renamez(ch *file1, ch *file2);                                              //  rename, also across file systems
 int  check_create_dir(ch *path);                                                 //  check if folder exists, ask to create
-int  copyFile(ch *sfile, ch *dfile);                                             //  copy file to file or file to folder
 int  cp_copy(ch *sfile, ch *dfile);                                              //  same, using shell "cp -f -p"
 uint diskspace(ch *file);                                                        //  get disk space for given file, MB
 ch * get_file_extension(ch *file);                                               //  get correct .ext if file.ext is wrong
@@ -154,33 +149,18 @@ int  zreaddir(ch *folder, ch **&files);
 int  zreadfile(ch *filename, ch **&rrecs);                                       //  read file, return array of records
 int  zwritefile(ch *filename, ch **&rrecs);                                      //  write array of records to file
 void zreadfile_free(ch **&rrecs);                                                //  free zreadfile() memory
-ch * combine_argvs(int argc, ch *argv[], int Nth);                               //  combine argv[ii] elements Nth to last
 
-//  measure CPU time spent in a function or code block within a function
-
-extern VOL double cpu_profile_timer;                                             //  internal data tables
-extern VOL double cpu_profile_table[100];
-extern VOL double cpu_profile_elapsed;
-void cpu_profile_init();                                                         //  initialize at start of test
-void cpu_profile_report();                                                       //  report CPU time per function
-inline void cpu_profile_enter(int fnum)                                          //  at entry to measured code block
-{  cpu_profile_timer = cpu_profile_elapsed;  }
-inline void cpu_profile_exit(int fnum)                                           //  at exit from measured code block
-{  cpu_profile_table[fnum] += cpu_profile_elapsed - cpu_profile_timer; }
-
-int pagefaultrate();                                                             //  monitor own process hard fault rate
-
-//  string macros and functions =================================================
-
-#define strmatch(str1,str2) (! strcmp((str1),(str2)))                            //  TRUE if strings equal
-#define strmatchN(str1,str2,cc) (! strncmp((str1),(str2),(cc)))                  //  TRUE if strings[cc] equal
-#define strmatchcase(str1,str2) (! strcasecmp((str1),(str2)))                    //  TRUE if strings equal, ignoring case
-#define strmatchcaseN(str1,str2,cc) (! strncasecmp((str1),(str2),(cc)))          //  TRUE if strings[cc] equal, ignoring case
+//  string functions ============================================================
 
+int strmatch(cch *str1, cch *str2);                                              //  true if strings equal, null strings allowed
+int strmatchN(cch *str1, cch *str2, int cc);                                     //  true if leading N characters equal
+int strmatchcase(cch *str1, cch *str2);                                          //  true if strings equal except for case
+int strmatchcaseN(cch *str1, cch *str2, int cc);                                 //  true if leading N chars equal except case
 ch * substringR(ch *string, ch *delims, int Nth);                                //  get Nth delimited substring, thread-safe
 ch * substringR(ch *string, ch delim, int Nth);                                  //  same, single delimiter
 ch * substring(ch *string, ch *delims, int Nth);                                 //  same, no zfree() needed, not thread-safe
 ch * substring(ch *string, ch delim, int Nth);                                   //  same, single delimiter
+int  get_substrings(ch *instr, ch dlm, int cnt, int maxcc, ch **outstr);         //  get all substrings from delimited string
 int  strHash(ch *string, uint max);                                              //  string --> random int 0 to max-1
 int  strncpy0(ch *dest, ch *source, uint cc);                                    //  strncpy, insure null, return 0 if fit
 void strnPad(ch *dest, ch *source, int cc);                                      //  strncpy with blank padding to cc
@@ -312,9 +292,9 @@ zlist_t * zlist_insert_zlist(zlist_t *zlist1, zlist_t *zlist2, int Nth);
 zlist_t * zlist_remove(zlist_t *zlist1, zlist_t *zlist2);                        //  remove all members of zlist2 from zlist1
 void zlist_sort(zlist_t *zlist);                                                 //  sort zlist ascending
 void zlist_sort(zlist_t *zlist, int ccfunc(ch *, ch *));                         //  sort zlist via caller compare function
-ch * zlist_choose(zlist_t *zlist);                                               //  popup menu, user choice of zlist member
 int zlist_to_file(zlist_t *zlist, ch *filename);                                 //  make file from zlist
 zlist_t * zlist_from_file(ch *filename);                                         //  make zlist from file
+zlist_t * zlist_from_folder(ch *foldername);                                     //  make zlist from files in a folder
 
 //  random number functions =====================================================
 
@@ -343,8 +323,7 @@ void Qtext_put(Qtext *qtext, ch *format, ...);
 ch * Qtext_get(Qtext *qtext);                                                    //  remove text string from Qtext queue
 void Qtext_close(Qtext *qtext);                                                  //  close Qtext, zfree() leftover strings
 
-//  compute variance for a list of numbers  
-//  std. deviation = sqrtf(variance) 
+//  compute variance for a list of numbers (std. deviation = sqrtf(variance))
 
 float variance(float *vals, int N);
 
@@ -402,34 +381,34 @@ void draw_context_destroy(draw_context_t &context);
 
 /********************************************************************************/
 
-//  textwidget functions - scrollable text widget for text reports and line editing
-//  widget = zdialog_gtkwidget(zd,textwidget) where textwidget is a zdialog "text" widget type
+//  txwidget functions - scrollable text widget for text reports and line editing
+//  widget = zdialog_gtkwidget(zd,txwidget) where txwidget is a zdialog "text" widget type
 
-void textwidget_clear(GtkWidget *widget);                                              //  clear all text
-void textwidget_clear(GtkWidget *widget, int line);                                    //  clear text from line to end
-int  textwidget_linecount(GtkWidget *widget);                                          //  get current line count
-void textwidget_append(GtkWidget *widget, int bold, ch *format, ...);                  //  append line
-void textwidget_append2(GtkWidget *widget, int bold, ch *format, ...);                 //  append line and scroll to end
-void textwidget_insert(GtkWidget *widget, int bold, int line, ch *format, ...);        //  insert line 
-void textwidget_replace(GtkWidget *widget, int bold, int line, ch *format, ...);       //  replace line
-void textwidget_delete(GtkWidget *widget, int line);                                   //  delete line
-int  textwidget_find(GtkWidget *widget, ch *matchtext, int line1, int hilite);         //  find matching line
-void textwidget_insert_pixbuf(GtkWidget *textwidget, int line, GdkPixbuf *pixbuf);     //  insert pixbuf image
-void textwidget_scroll(GtkWidget *widget, int line);                                   //  scroll line on screen
-void textwidget_scroll_top(GtkWidget *widget, int line);                               //  scroll line to top of window
-void textwidget_get_visible_lines(GtkWidget *textwidget, int &top, int &bott);         //  get range of visible lines
-void textwidget_dump(GtkWidget *widget, ch *filename);                                 //  dump all text into a file
-void textwidget_save(GtkWidget *widget, GtkWindow *parent);                            //  same, with save-as dialog
-ch * textwidget_line(GtkWidget *widget, int line, int strip);                          //  retrieve line (strip \n) 
-void textwidget_highlight_line(GtkWidget *widget, int line);                           //  highlight line
-ch * textwidget_word(GtkWidget *, int line, int posn, ch *dlims, ch &end);             //  retrieve word
-void textwidget_highlight_word(GtkWidget *widget, int line, int posn, int cc);         //  highlight word
-void textwidget_bold_word(GtkWidget *widget, int line, int posn, int cc);              //  make word bold
-void textwidget_underline_word(GtkWidget *widget, int line, int posn, int cc);         //  make word underlined
-void textwidget_font_attributes(GtkWidget *widget);                                    //  set font attributes for all text
+void txwidget_clear(GtkWidget *widget);                                          //  clear all text
+void txwidget_clear(GtkWidget *widget, int line);                                //  clear text from line to end
+int  txwidget_linecount(GtkWidget *widget);                                      //  get current line count
+void txwidget_append(GtkWidget *widget, int bold, ch *format, ...);              //  append line
+void txwidget_append2(GtkWidget *widget, int bold, ch *format, ...);             //  append line and scroll to end
+void txwidget_insert(GtkWidget *widget, int bold, int line, ch *format, ...);    //  insert line 
+void txwidget_replace(GtkWidget *widget, int bold, int line, ch *format, ...);   //  replace line
+void txwidget_delete(GtkWidget *widget, int line);                               //  delete line
+int  txwidget_find(GtkWidget *widget, ch *matchtext, int line1, int hilite);     //  find matching line
+void txwidget_insert_pixbuf(GtkWidget *widget, int line, GdkPixbuf *pixbuf);     //  insert pixbuf image
+void txwidget_scroll(GtkWidget *widget, int line);                               //  scroll line on screen
+void txwidget_scroll_top(GtkWidget *widget, int line);                           //  scroll line to top of window
+void txwidget_get_visible_lines(GtkWidget *widget, int &top, int &bott);         //  get range of visible lines
+void txwidget_dump(GtkWidget *widget, ch *filename);                             //  dump all text into a file
+void txwidget_save(GtkWidget *widget, GtkWindow *parent);                        //  same, with save-as dialog
+ch * txwidget_line(GtkWidget *widget, int line, int strip);                      //  retrieve line (strip \n) 
+void txwidget_highlight_line(GtkWidget *widget, int line);                       //  highlight line
+ch * txwidget_word(GtkWidget *, int line, int posn, ch *dlims, ch &end);         //  retrieve word
+void txwidget_highlight_word(GtkWidget *widget, int line, int posn, int cc);     //  highlight word
+void txwidget_bold_word(GtkWidget *widget, int line, int posn, int cc);          //  make word bold
+void txwidget_underline_word(GtkWidget *widget, int line, int posn, int cc);     //  make word underlined
+void txwidget_font_attributes(GtkWidget *widget);                                //  set font attributes for all text
 
-typedef void textwidget_callbackfunc_t(GtkWidget *, int line, int posn, int KBkey);    //  widget event function to receive
-void textwidget_set_eventfunc(GtkWidget *, textwidget_callbackfunc_t func);            //    mouse click and KB events
+typedef int txwidget_CBfunc_t(GtkWidget *, int line, int posn, ch *input);       //  widget event function to receive
+void txwidget_set_eventfunc(GtkWidget *, txwidget_CBfunc_t func);                //    mouse click and KB events
 
 
 /********************************************************************************/
@@ -453,10 +432,11 @@ int stbar_message(GtkWidget *stbar, ch *message);
 /********************************************************************************/
 
 GtkWidget * create_popmenu();                                                    //  create an empty popup menu
-GtkWidget * add_popmenu_item(GtkWidget *popmenu, ch *mname,                      //  add menu item to popup menu
-                           cbFunc func, ch *arg, ch *mtip = 0);
+void add_popmenu(GtkWidget *popmenu, ch *item, cbFunc func, ch *arg, ch *tip);   //  add menu item to popup menu
 void popup_menu(GtkWidget *, GtkWidget *popmenu);                                //  pop-up menu at current mouse posn.
 
+ch * popup_choose(zlist_t *zlist);                                               //  popup to choose item from list
+ch * popup_choose(ch *file);                                                     //  same, list from text file
 
 /********************************************************************************/
 
@@ -492,9 +472,8 @@ struct Vmenu {
 };
 
 Vmenu *Vmenu_new(GtkWidget *vbox, float fgRGB[3], float bgRGB[3]);               //  create new menu in parent vbox
-void Vmenu_add(Vmenu *vbm, ch *name, ch *icon,                                   //  add menu item with response function
-               int iconww, int iconhh, ch *desc,                                 //  function may be popup_menu()
-               cbFunc func, ch *arg);
+void Vmenu_add(Vmenu *vbm, ch *name, ch *icon, int iconww, int iconhh,           //  add menu item with response function
+                                       ch *desc, cbFunc func, ch *arg);          //  function may be popup_menu()
 void Vmenu_add_setup(Vmenu *vbm, int me, cbFunc RMfunc, ch *arg);                //  add opt. setup function
 void Vmenu_add_RMfunc(Vmenu *vbm, int me, cbFunc RMfunc, ch *arg);               //  add function for right mouse click
 void Vmenu_block(int flag);                                                      //  block or unblock menu
@@ -616,7 +595,6 @@ int zdialog_rescale(zdialog *zd, ch *wname, float, float, float);
 
 typedef int zdialog_event(zdialog *zd, ch *wname);                               //  widget event callback function
 int zdialog_run(zdialog *zd, zdialog_event = 0, ch *posn = 0);                   //  run dialog, handle events
-void zdialog_KB_addshortcut(ch *key, ch *menu);                                  //  KB shortcut for zdialog compl. button
 void KBevent(GdkEventKey *event);                                                //  extern: pass KB events to main app
 
 int zdialog_send_event(zdialog *zd, ch *event);                                  //  send an event to an active dialog
@@ -664,7 +642,7 @@ int  zdialog_edit_textfile(GtkWidget *parent, char *file);
 //  write text report in popup window
 
 zdialog * popup_report_open(ch *title, GtkWidget *parent, int ww, int hh,        //  open popup report - pixel size,
-               int wrap, int Fheader, textwidget_callbackfunc_t CBfunc, ...);    //    wrap, header line, callback function
+               int wrap, int Fheader, txwidget_CBfunc_t CBfunc, ...);            //    wrap, header line, callback function
 void popup_report_header(zdialog *zd, int bold, ch *format, ...);                //  write non-scrolling header line
 void popup_report_write(zdialog *zd, int bold, ch *format, ...);                 //  write text line
 void popup_report_write2(zdialog *zd, int bold, ch *format, ...);                //  write text line and scroll to end
@@ -689,22 +667,21 @@ void popup_report_bold_word(zdialog *zd, int line, int posn, int cc);
 void popup_report_font_attributes(zdialog *zd);                                  //  font attributes for entire report
 void popup_report_close(zdialog *zd, int secs);                                  //  close window after seconds
 
-//  shell command to popup window
+int popup_command(ch *cmd, int ww=400, int hh=300, GtkWidget *p=0, int top=0);   //  shell command to popup window
+int  monitor_file(ch *file);                                                     //  monitor a text file in a popup window
 
-int popup_command(ch *command, int ww = 400, int hh = 300, GtkWidget *parent = 0, int top = 0);
-
-//  popups: message, dialog, image, picklist
+//  popups: message, dialog, image
 
+ch * boldred(ch *message);                                                       //  convert dialog message to bold/red font
+void zmessageACK_init();                                                         //  initz. for thread usage
 void zmessageACK(GtkWidget *parent, ch *format, ... );                           //  display message, wait for OK
 int  zmessageYN(GtkWidget *parent, ch *format, ... );                            //  display message, wait for YES/NO
 zdialog * zmessage_post(GtkWidget *, ch *loc, int s, ch *f, ...);                //  show message, timeout or cancel
 
 void poptext_screen(ch *text, int px, int py, float s1, float s2);               //  show popup text at screen posn
 void poptext_mouse(ch *text, int dx, int dy, float s1, float s2);                //  same, at mouse posn + offset
-void poptext_window(GtkWindow *, ch *tx, int x, int y, float s1, float s2);      //  same, at window posn + offset
 void poptext_widget(GtkWidget *, ch *tx, int x, int y, float s1, float s2);      //  same, at widget posn + offset
 int  poptext_killnow();                                                          //  kill current popup window
-ch * popup_picklist(GtkWidget *paren, ch **list, ch **desc, int Np);             //  popup picklist, return choice
 
 //  file chooser dialogs for one file, multiple files, or folder
 
@@ -727,7 +704,6 @@ void drag_drop_dest(GtkWidget *window, drag_drop_dest_func);
 
 PIXBUF * get_thumbnail(ch *fpath, int size);                                     //  get sized thumbnail for image file
 GdkCursor * zmakecursor(ch *iconfile);                                           //  make a cursor from an image file
-PIXBUF * gdk_pixbuf_rotate(PIXBUF *, float deg, int alfa = 0);                   //  rotate pixbuf through any angle
 PIXBUF * gdk_pixbuf_stripalpha(PIXBUF *pixbuf);                                  //  strip alpha channel from pixbuf
 PIXBUF * text_pixbuf(ch *text, ch *font, int fsize, GtkWidget *);                //  create pixbuf with text using font
 
